/**
 * 
 * File: FuzzyKNN.java
 * 
 * The Fuzzy K-NN algorithm. 
 * 
 * @author Written by Joaqun Derrac (University of Granada) 21/06/2011 
 * @version 1.0 
 * @since JDK1.5
 * 
 */

package keel.Algorithms.FingerprintClassification.Hong08;

import java.util.Arrays;
import java.util.StringTokenizer;

import org.core.Files;
import org.core.Randomize;

import keel.Algorithms.FingerprintClassification.FingerprintClassificationAlgorithm;
import keel.Algorithms.FingerprintClassification.ReportTool;
import keel.Algorithms.FingerprintClassification.Timer;


public class Hong08 extends FingerprintClassificationAlgorithm {
	
	private static final int MODE_NB = 0;
	private static final int MODE_SVM = 1;
	private static final int MODE_COMB = 2;

	private int mode=1;

	
	private int [][][] AVC;
	private int [][] AV;
	private static int [] numValues;
	private static int maxNumValues;
	private int [] classCount;
	double [] outputNB;
	
	private double C;
	private double gamma;
	
	private SMOWrapper SVM;
	
	private static int NB_ATTS=10;

	/** 
	 * Reads the parameters of the algorithm. 
	 * 
	 * @param script Configuration script
	 * 
	 */
	@Override
	protected void readParameters(String script) {
		
		String file;
		String line;
		String type;
		StringTokenizer fileLines, tokens;
		
	    file = Files.readFile (script);
	    fileLines = new StringTokenizer (file,"\n\r");
	    
	    //Discard in/out files definition
	    fileLines.nextToken();
	    fileLines.nextToken();
	    fileLines.nextToken();
	    
	    //Getting the C parameter
	    line = fileLines.nextToken();
	    tokens = new StringTokenizer (line, "=");
	    tokens.nextToken();
	    C = Double.parseDouble(tokens.nextToken().substring(1));
	    
	    //Getting the C parameter
	    line = fileLines.nextToken();
	    tokens = new StringTokenizer (line, "=");
	    tokens.nextToken();
	    gamma = Double.parseDouble(tokens.nextToken().substring(1));
	    
	    //Getting the classifier mode
	    line = fileLines.nextToken();
	    tokens = new StringTokenizer (line, "=");
	    tokens.nextToken();
	    type = tokens.nextToken().substring(1); 
	    
	    mode=MODE_SVM;

	    if(type.equalsIgnoreCase("MODE_NB")){
	    	mode=MODE_NB;
	    }
	    if(type.equalsIgnoreCase("MODE_COMB")){
	    	mode=MODE_COMB;
	    }
	    
	    /*//Getting the K parameter
	    line = fileLines.nextToken();
	    tokens = new StringTokenizer (line, "=");
	    tokens.nextToken();
	    k = Integer.parseInt(tokens.nextToken().substring(1));
	    
	    
	    //Getting the learning rate
	    line = fileLines.nextToken();
	    tokens = new StringTokenizer (line, "=");
	    tokens.nextToken();
	    learning = Double.parseDouble(tokens.nextToken().substring(1));*/
  
	    
	} //end-method	

	/**
	 * Main builder. Initializes the methods' structures
	 * 
	 * @param script Configuration script
	 */
	public Hong08(String script){
		
		readDataFiles(script);
		
		//Naming the algorithm
		name="Hong08";
		
		classCount=new int [nClasses];
		
		numValues=new int [NB_ATTS];

		Arrays.fill(numValues,0);
		maxNumValues=1;
		
		numValues[0]=3;
		numValues[1]=3;
		numValues[2]=6;
		numValues[3]=6;
		numValues[4]=5;
		numValues[5]=5;
		numValues[6]=5;
		numValues[7]=5;
		numValues[8]=3;
		numValues[9]=3;
		
		maxNumValues=6;
			
		AVC=new int[NB_ATTS][maxNumValues][nClasses];
		AV=new int[NB_ATTS][maxNumValues];

		SVM= new SMOWrapper(C,gamma,12345);

		//Initialization of random generator    
	    Randomize.setSeed(seed);

	    //Initialization of Reporting tool
	    ReportTool.setOutputFile(outFile[2]);
	    
	    Timer.resetTime();
	    
	} //end-method	
	
	private int discretize(double value, int att){
		
		int discrete=0;
		switch(att){
		
		case 0:
		case 1:
		case 8:
		case 9:
			if(value<0.4){
				discrete=0;
			}else if (value<0.9){
				discrete=1;
			}else{
				discrete=2;
			}
		break;
		case 2:
		case 3:
			if(value<0.1){
				discrete=0;
			}else if (value<0.3){
				discrete=1;
			}else if (value<0.5){
				discrete=2;
			}else if (value<0.7){
				discrete=3;
			}else if (value<0.9){
				discrete=4;
			}else{
				discrete=5;
			}
		break;
		case 4:
		case 5:
		case 6:
		case 7:
			if(value<0.2){
				discrete=0;
			}else if (value<0.4){
				discrete=1;
			}else if (value<0.6){
				discrete=2;
			}else if (value<0.8){
				discrete=3;
			}else{
				discrete=4;
			}
		break;
		}
		
		
		return discrete;
	}
	
	private void calcProbabilities(){
		
		int output;
		int nInputs=NB_ATTS;
		int value;
		
		//calcular probabilidades de clases
		
		Arrays.fill(classCount,0);
			
		for(int i=0;i<trainOutput.length;i++){
			classCount[trainOutput[i]]++;
		}
		
		//calcular AVC
		
		for(int i=0;i<nInputs;i++){
			for(int j=0;j<maxNumValues;j++){
				for(int k=0;k<nClasses;k++){
					AVC[i][j][k]=0;
				}
				AV[i][j]=0;
			}			
		}
		
		
		for(int i=0;i<trainData.length;i++){
			output=trainOutput[i];
			for(int j=0;j<NB_ATTS;j++){
				value=discretize(trainData[i][j],j);
				AVC[j][value][output]++;
				AV[j][value]++;
			}
			
		}
		
	}
	
	public void printAVC(){
		
		String cadena;
		
		for(int i=0;i<NB_ATTS;i++){
			cadena="";
			for(int j=0;j<numValues[i];j++){
				cadena+=" ";
				for(int k=0;k<nClasses;k++){
					cadena+=AVC[i][j][k]+"*";
				}
			}
			System.out.println(cadena+" "+i );
		}
	}	
	
	/**
	 * Generates the model of the algorithm
	 */
	public void generateModel (){
		
		if(mode!=MODE_SVM){
			//compute probabilities for the NB
			calcProbabilities();
		}
		
		if(mode!=MODE_NB){
			try{
				SVM.generateTrainingFiles(trainFile);
				SVM.generateTestFiles(testFile);
			}catch(Exception e){}
		}
		
		
		//End of model time
		Timer.setModelTime();
	
		//Showing results
		System.out.println(name+" "+ relation + " Model " + Timer.getModelTime() + "s");
		
	}
	
	
	/**
	 * Classifies the training set (leave-one-out)
	 */
	public void classifyTrain(){
	    
		//Start of training time
		Timer.resetTime();
		
		classifyTrainSet();
		
		//End of training time
		Timer.setTrainingTime();
		
		//Showing results
		System.out.println(name+" "+ relation + " Training " + Timer.getTrainingTime() + "s");
		
	} //end-method	
	
	/**
	 * Classifies the test set
	 */
	public void classifyTest(){
		    
		//Start of training time
		Timer.resetTime();
		
		classifyTestSet();
		
		//End of test time
		Timer.setTestTime();
		
		//Showing results
		System.out.println(name+" "+ relation + " Test " + Timer.getTestTime() + "s");	
		
	} //end-method	
	
	/**
	 * Classifies the training set
	 */
	public void classifyTrainSet(){
				
		SVM.classify();
		
		for(int i=0;i<trainData.length;i++){
			
			trainPrediction[i]=classifyInstance(i,true);
			
		}

	} //end-method	
	
	/**
	 * Classifies the test set
	 */
	public void classifyTestSet(){

		for(int i=0;i<testData.length;i++){
			
			testPrediction[i]=classifyInstance(i,false);
			
		}

	} //end-method	
	
	private int classifyInstance(int index, boolean training){
		
		int output=-1;
		int discrete[];
		double [] SVMoutput;
		double sumClasses [];
		double sorted [];
		int pointer;
		
		int selected;
		double max;
		
		discrete=new int [NB_ATTS];
		
		for(int i=0;i<NB_ATTS;i++){
			if(training){
				discrete[i]=discretize(trainData[index][i],i);
			}
			else{
				discrete[i]=discretize(testData[index][i],i);
			}
		}
		
		sumClasses= new double [nClasses];
		
		Arrays.fill(sumClasses, 0.0);
		
		if(mode==MODE_NB){
			
			computeNB(discrete);
			
			
			//choose output
			selected=-1;
			max=0;
			
			for(int j=0;j<nClasses;j++){
				if(max<=outputNB[j]){
					max=outputNB[j];
					selected=j;
				}
			}

			return selected;
		}
		
		if(mode==MODE_SVM){
			
			if(!training){
				SVMoutput=SVM.classifyTestInstance(index);
				output=maxV(SVMoutput);
			}
			else{
				SVMoutput=SVM.classifyTrainInstance(index);
				output=maxV(SVMoutput);
			}
			
		}
		
		if(mode==MODE_COMB){
			
			if(!training){
				
				SVMoutput=SVM.classifyTestInstance(index);			
				computeNB(discrete);
				sorted= new double [nClasses];
				System.arraycopy(outputNB, 0, sorted, 0, nClasses);
				
				pointer=1;
				while(pointer<=nClasses){
					for(int k=0;k<nClasses;k++){
						if((outputNB[k]==sorted[nClasses-pointer])&&(SVMoutput[k]>(1.0/(double)nClasses))){
							return k;
						}
					}
					pointer++;
				}
				
				return -1;
				
				/*max=SVMoutput[0]*outputNB[0];
				output=0;
				System.out.println("***********"+index+"\n"+SVMoutput[0]+" "+outputNB[0]);
				for(int k=1;k<nClasses;k++){
					System.out.println(SVMoutput[k]+" "+outputNB[k]);
					if(max<SVMoutput[k]*outputNB[k]){
						max=SVMoutput[k]*outputNB[k];
						output=k;
					}
				}*/
				
			
			}
			else{
				SVMoutput=SVM.classifyTrainInstance(index);			
				computeNB(discrete);
				sorted= new double [nClasses];
				System.arraycopy(outputNB, 0, sorted, 0, nClasses);
				
				pointer=1;
				while(pointer<=nClasses){
					for(int k=0;k<nClasses;k++){
						if((outputNB[k]==sorted[nClasses-pointer])&&(SVMoutput[k]>(1.0/(double)nClasses))){
							return k;
						}
					}
					pointer++;
				}
				
				return -1;
			}
			
		}
		
		return output;
	}
	
	private int maxV(double a[]){
	
		double max=a[0];
		int out=0;
		
		for(int i=1;i<a.length;i++){
			if(a[i]>max){
				max=a[i];
				out=i;
			}
		}
		
		return out;
	}
	
	private void computeNB(int [] example){
		
		outputNB=new double[nClasses];
		
		Arrays.fill(outputNB, 1.0);
		
		//multiplicar atributos
		for(int j=0;j<example.length;j++){
			for(int k=0;k<nClasses;k++){
				//Laplace correction
				outputNB[k]*=(double)((double)(AVC[j][example[j]][k]+1.0)/(double)(AV[j][example[j]]+nClasses));
			}
		}
		
		//multiplicar clases
		for(int j=0;j<nClasses;j++){
			outputNB[j]*=(double)((double)classCount[j]/(double)trainData.length);
		}
		
	}
	
	
	/**
	 * Reports the results obtained
	 */
	public void printReport(){
		
		writeOutput(outFile[0], trainOutput, trainPrediction);
		writeOutput(outFile[1], testOutput, testPrediction);
		
		ReportTool.setResults(trainOutput,trainPrediction,testOutput,testPrediction,nClasses);
		
		ReportTool.printReport();
		
	}
} //end-class 
