/**
 * 
 * File: FuzzyKNN.java
 * 
 * The Fuzzy K-NN algorithm. 
 * 
 * @author Written by Joaqun Derrac (University of Granada) 21/06/2011 
 * @version 1.0 
 * @since JDK1.5
 * 
 */

package keel.Algorithms.FingerprintClassification.Hong08;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;

import keel.Algorithms.FingerprintClassification.Hong08.SMO.SMO;

public class SMOWrapper{
	
	private static String nameTrainNew;
	private static String nameTestNew;

	private SMO classifier;
	
	public SMOWrapper(double C,double RBFGamma,int Seed){

    	classifier = new SMO(Seed,C,RBFGamma);
	}
	
	public void generateTrainingFiles(String trainFile) throws IOException{
		
		//prepare input data
    	FileOutputStream out;
        PrintStream p;
        File file; 
        
        //create the new files
        
        nameTrainNew="input_train.dat";
        
        out = new FileOutputStream(nameTrainNew);
        p = new PrintStream(out);
        file = new File(trainFile);
        
        FileInputStream fstream = new FileInputStream(file);
        DataInputStream in = new DataInputStream(fstream);
        BufferedReader br = new BufferedReader(new InputStreamReader(in));
        
        //NEW TRAINING DATA:
        //print the header:
        String line = br.readLine();
        while(line.contains("@")&&(!line.contains("Class"))){
            p.println(line);
            line=br.readLine();
        }
        
        //print class line 
        p.println(line);
        line=br.readLine();

        p.println(line);     
        line=br.readLine();
        
        while(line.contains("@")){
            p.println(line);
            line=br.readLine();
        }
        //print the instances:

        while(line!=null){
            p.println(line);
            line=br.readLine();
        }
        
        p.close();
        br.close();
        in.close();
        out.close();
        fstream.close();
	}
	
	public void generateTestFiles(String testFile){
		
		try{
		nameTestNew="input_test.dat";
        
        //NEW TESTING DATA: (=old training data)  
        FileOutputStream out;
        PrintStream p;
        File file; 
  
        file = new File(testFile);
    
        FileInputStream fstream = new FileInputStream(file);
        DataInputStream in = new DataInputStream(fstream);
        BufferedReader br = new BufferedReader(new InputStreamReader(in));
        
        out = new FileOutputStream(nameTestNew);
        p = new PrintStream(out);
        
        //print the header:
        String line = br.readLine();
        while(line.contains("@")&&(!line.contains("Class"))){
            p.println(line);
            line=br.readLine();
        }
        
        //print class line 
        p.println(line);
        line=br.readLine();

        p.println(line);     
        line=br.readLine();
        
        while(line.contains("@")){
            p.println(line);
            line=br.readLine();
        }

        while(line!=null){
        	p.println(line);
        	line=br.readLine();

        }
        //print test instance
        p.close();
        br.close();
        in.close();
        out.close();
        fstream.close();
		}catch(Exception e){
			
		}
	}
	
	public void classify(){
		
		classifier.runModel();

	}
	
	public double [] classifyTestInstance(int index){
		return classifier.getTestRealOutput(index);
	}
	
	public double [] classifyTrainInstance(int index){
		return classifier.getTrainRealOutput(index);
	}
}