package keel.Algorithms.FingerprintClassification.Jain99;

import org.core.Randomize;

public class Jain99NN {
	
	private Neuron inputs [];
	private Neuron hidden [];
	private Neuron outputs [];
	
	private int nInput;
	private int nHidden;
	private int nOutput;
	
	private double inputWeights [][];
	private double hiddenWeights [][];
	
	private double inputWeightsMomentum [][];
	private double hiddenWeightsMomentum [][];
		
	private final double BIAS = -1;
	private final double EPSILON = 0.00000001;
	
	private double LEARNING_RATE;
	private double MOMENTUM;
	
	public Jain99NN(int inputAtt,int nClasses,int hidden,double learning,double momen){
		
		nInput=inputAtt;
		nHidden=hidden;
		nOutput=nClasses;
		
		LEARNING_RATE=learning;
		
		MOMENTUM=momen;
		
		initialize();
	}
	
	public double train(double example [], int output){
		
		double NNoutputs[];
		double error, aux;
		double desiredOutputs[];
		
		double aOut,aHidden;
		double partialDerivative;
		
		double deltaWeight, newWeight;
		double sumKoutputs;
		
		error=0.0;
		
		NNoutputs=forward(example);
		
		desiredOutputs= new double [outputs.length];
		
		for(int i=0;i<outputs.length;i++){
			
			if(i==output){				
				desiredOutputs[i]=1.0-EPSILON;		
			}
			else{
				desiredOutputs[i]=EPSILON;
			}
			
			aux=Math.pow(desiredOutputs[i] - NNoutputs[i] , 2);	
			error+=aux;
		}
		
		//backpropagation (hidden => output)
		for(int i=0;i<hidden.length;i++){
			for(int j=0;j<outputs.length;j++){
				aOut = outputs[j].getOutput();
				aHidden = hidden[j].getOutput();
				partialDerivative = -aOut * (1 - aOut) * aHidden * (desiredOutputs[j] - aOut);
				deltaWeight = -LEARNING_RATE * partialDerivative;
				newWeight =  hiddenWeights[i][j]+ deltaWeight;
				hiddenWeights[i][j] = newWeight + (MOMENTUM* hiddenWeightsMomentum[i][j]);
				hiddenWeightsMomentum[i][j]=deltaWeight;
			}
		}
			
		//bias unit
		for(int j=0;j<outputs.length;j++){
			aOut = outputs[j].getOutput();
			aHidden = hidden[j].getOutput();
			partialDerivative = -aOut * (1 - aOut) * aHidden * (desiredOutputs[j] - aOut);
			deltaWeight = -LEARNING_RATE * partialDerivative;
			newWeight =  hiddenWeights[hidden.length][j]+ deltaWeight;
			hiddenWeights[hidden.length][j] = newWeight + (MOMENTUM* hiddenWeightsMomentum[hidden.length][j]);
			hiddenWeightsMomentum[hidden.length][j]=deltaWeight;
		}

		//backpropagation (input => hidden)
		
		for(int i=0;i<inputs.length;i++){
			for(int j=0;j<hidden.length;j++){
				
				sumKoutputs = 0;
				for(int l=0;l<outputs.length;l++){
					sumKoutputs += -(desiredOutputs[l] - outputs[l].getOutput()) * outputs[l].getOutput() * (1 - outputs[l].getOutput()) * hiddenWeights[j][l];
				}
				
				partialDerivative = hidden[j].getOutput() * (1 - hidden[j].getOutput()) * inputs[i].getOutput() * sumKoutputs;
				deltaWeight = -LEARNING_RATE * partialDerivative;
				newWeight =  inputWeights[i][j]+ deltaWeight;
				inputWeights[i][j] = newWeight + (MOMENTUM* inputWeightsMomentum[i][j]);
				inputWeightsMomentum[i][j]=deltaWeight;
			}
		}
					
		//bias unit
		for(int j=0;j<hidden.length;j++){
			
			sumKoutputs = 0;
			for(int l=0;l<outputs.length;l++){
				sumKoutputs += -(desiredOutputs[l] - outputs[l].getOutput()) * outputs[l].getOutput() * (1 - outputs[l].getOutput()) * hiddenWeights[j][l];
			}
			
			partialDerivative = hidden[j].getOutput() * (1 - hidden[j].getOutput()) * -1 * sumKoutputs;
			deltaWeight = -LEARNING_RATE * partialDerivative;
			newWeight =  inputWeights[inputs.length][j]+ deltaWeight;
			inputWeights[inputs.length][j] = newWeight + (MOMENTUM* inputWeightsMomentum[inputs.length][j]);
			inputWeightsMomentum[inputs.length][j]=deltaWeight;
		}
				
		return error;
	}
	
	public double validate(double example [], int output){
		
		double outputs[];
		double error, aux;
		
		error=0.0;
		
		outputs=forward(example);
		
		for(int i=0;i<outputs.length;i++){
			
			if(i==output){
				aux=Math.pow((1.0-EPSILON) - outputs[i], 2);
			}
			else{
				aux=Math.pow((EPSILON) - outputs[i], 2);
			}
			
			error+=aux;
		}
		
		return error;
		
	}
	
	public int classify(double example []){
		
		double outputs[];
		int output;
		double value;
		
		outputs=forward(example);
		
		output=0;
		value=outputs[0];
		
		System.out.println(outputs[0]);
		for(int i=1;i<nOutput;i++){
			if(value<outputs[i]){
				System.out.println(outputs[i]);
				value=outputs[i];
				output=i;
			}
		}
		System.out.println("**************"+output);
		
		return output;
	}
	
	private void initialize(){
		
		//initialize neurons
		inputs = new Neuron [nInput];
		
		for(int i=0; i < nInput; i++){
			inputs[i] = new Neuron(Neuron.INPUT);
		}

		hidden = new Neuron [nHidden];
		
		for(int i=0; i < nHidden; i++){
			hidden[i] = new Neuron(Neuron.HIDDEN);
		}
		
		outputs = new Neuron [nOutput];
		
		for(int i=0; i < nOutput; i++){
			outputs[i] = new Neuron(Neuron.OUTPUT);
		}

		//initialize weights
		
		inputWeights= new double [nInput+1][nHidden]; //bias unit is added	
		hiddenWeights= new double [nHidden+1][nOutput]; //bias unit is added
		
		inputWeightsMomentum= new double [nInput+1][nHidden]; //bias unit is added	
		hiddenWeightsMomentum= new double [nHidden+1][nOutput]; //bias unit is added
		
		//random initialization
		for(int i=0; i < nInput+1; i++){
			for(int j=0; j < nHidden; j++){
				inputWeights[i][j]=((Randomize.Rand()*2.0)-1.0); //[-1,1]
				inputWeightsMomentum[i][j]=0.0;
			}
		}
		
		for(int i=0; i < nHidden+1; i++){
			for(int j=0; j < nOutput; j++){
				hiddenWeights[i][j]=((Randomize.Rand()*2.0)-1.0); //[-1,1]
				hiddenWeightsMomentum[i][j]=0.0;
			}
		}
		
		System.out.println("Network initialized!");
	}
	
	private double[] forward(double example []){
		
		double [] fOutputs;
		double s;
		
		//input layer
		
		for(int j=0;j<inputs.length;j++){
					
			inputs[j].setOutput(example[j]);	
		}
		
		//hidden layer
		for(int i=0;i<hidden.length;i++){
			
			s=0;
			for(int j=0;j<inputs.length;j++){
			
				s+=inputs[j].getOutput()*inputWeights[j][i];
			
			}
			
			s+=BIAS*inputWeights[inputs.length][i]; //bias
			
			hidden[i].computeOutput(s);
		}
				
		//output layer
		for(int i=0;i<outputs.length;i++){
			
			s=0;
			for(int j=0;j<hidden.length;j++){
			
				s+=hidden[j].getOutput()*hiddenWeights[j][i];
			
			}
			
			s+=BIAS*hiddenWeights[hidden.length][i]; //bias
			
			outputs[i].computeOutput(s);
			
		}
		
		fOutputs=new double [outputs.length];
		
		for(int i=0;i<outputs.length;i++){
			fOutputs[i]=outputs[i].getOutput();
		}
		
		
		return fOutputs;
	}

}
